#!/usr/bin/env python

"""
Mail preparation support.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.config import MESSAGE_SENDER, OUTGOING_PREFIX
from email.mime.message import MIMEMessage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from smtplib import LMTP, SMTP

MESSAGE_SUBJECT = "Calendar system message"

MESSAGE_TEXT = """\
This is a message from the calendar system.
"""

PREAMBLE_TEXT = """\
This message contains several different parts, one of which will contain
calendar information that will only be understood by a suitable program.
"""

class Messenger:

    "Sending of outgoing messages."

    def __init__(self, sender=None, subject=None, body_text=None, preamble_text=None):
        self.sender = sender or MESSAGE_SENDER
        self.subject = subject or MESSAGE_SUBJECT
        self.body_text = body_text or MESSAGE_TEXT
        self.preamble_text = preamble_text or PREAMBLE_TEXT

    def sendmail(self, recipients, data, sender=None, outgoing_bcc=None, lmtp_socket=None):

        """
        Send a mail to the given 'recipients' consisting of the given 'data',
        using the given 'sender' identity if indicated, indicating an
        'outgoing_bcc' identity if indicated, delivering to a local mail system
        using LMTP if 'lmtp_socket' is provided.
        """

        if lmtp_socket:
            smtp = LMTP(lmtp_socket)
        else:
            smtp = SMTP("localhost")

        if outgoing_bcc:
            recipients = list(recipients) + ["%s+%s" % (OUTGOING_PREFIX, outgoing_bcc)]

        smtp.sendmail(sender or self.sender, recipients, data)
        smtp.quit()

    def make_outgoing_message(self, parts, recipients, sender=None, outgoing_bcc=None):

        """
        Make a message from the given 'parts' for the given 'recipients', using
        the given 'sender' identity if indicated, indicating an 'outgoing_bcc'
        identity if indicated.
        """

        message = self._make_summary_for_parts(parts)

        message["From"] = sender or self.sender
        for recipient in recipients:
            message["To"] = recipient
        if outgoing_bcc:
            message["Bcc"] = "%s+%s" % (OUTGOING_PREFIX, outgoing_bcc)
        message["Subject"] = self.subject

        return message

    def make_summary_message(self, msg, parts):

        """
        Return a simple summary using details from 'msg' and the given 'parts'.
        """

        message = self._make_summary_for_parts(parts)
        self._copy_headers(message, msg)
        return message

    def wrap_message(self, msg, parts):

        "Wrap 'msg' and provide the given 'parts' as the primary content."

        message = self._make_container_for_parts(parts)
        payload = message.get_payload()
        payload.append(MIMEMessage(msg))
        self._copy_headers(message, msg)
        return message

    def _make_summary_for_parts(self, parts):

        "Return a simple summary for the given 'parts'."

        if len(parts) == 1:
            return parts[0]
        else:
            return self._make_container_for_parts(parts)

    def _make_container_for_parts(self, parts):

        "Return a container for the given 'parts'."

        message = MIMEMultipart("mixed", _subparts=parts)
        message.preamble = self.preamble_text
        return message

    def _copy_headers(self, message, msg):
        message["From"] = msg["From"]
        message["To"] = msg["To"]
        message["Subject"] = msg["Subject"]

# vim: tabstop=4 expandtab shiftwidth=4
